//-----------------------------------------------------------------------------
// morse.c - ver.1.0
// Modul pro vysilani Morseovy abecedy
// leden 2007
// WinAVR-20060421
// OK1CTR@seznam.cz http://home.zcu.cz/~rlinhart
//-----------------------------------------------------------------------------

#include <inttypes.h>
#include <string.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "morse.h"

//#define _DEBUG_

// --- PROMENNE ----------------------------------------------------------------

// menitelne parametry
uint8_t morse_unit = UNIT;
uint8_t morse_paris = PARIS;

// rizeni pipadla
volatile uint8_t m_flag = 0;

// --- KONSTANTNI POLE VE FLASH ------------------------------------------------

// pismena, znak od MSB, 2 LSB bity delka - 1
prog_uint8_t morse_c[] = {
// A     B      C     D     E     F
   0x41, 0x83, 0xA3, 0x82, 0x00, 0x23,
   
// G     H      I     J     K     L
   0xC2, 0x03, 0x01, 0x73, 0xA2, 0x83,
   
// M     N      O     P     Q     R
   0xC1, 0x81, 0xE2, 0x63, 0xD3, 0x42,
   
// s     T      U     V     W     X
   0x02, 0x80, 0x22, 0x13, 0x62, 0x93,
   
// Y     Z
   0XB3, 0XC3
};

// cislice, znak od MSB, konst. delka 5 znacek
prog_uint8_t morse_n[] = {
// 0     1     2      3     4
   0xF8, 0x78, 0x38, 0x18, 0x08,
   
// 5     6     7      8     9
   0x00, 0x80, 0xC0, 0xE0, 0xF0
};

// znaky, pole textoveho vyjadreni
prog_uint8_t morse_zn[] = {
    '.',  ',',  '?',  '-',  '/',  '+',  '@',  '_'
};

// znaky, pole morse kodu, od MSB, LSB je delka 0=5, 1=6 zn.
prog_uint8_t morse_zc[] = {
   0x66, 0xCD, 0x31, 0x85, 0x90, 0x50, 0x15, 0x88
};

// --- INTERRUPTY --------------------------------------------------------------

// preteceni casovace 0
ISR (TIMER0_OVF_vect)
{
  static uint8_t beep = 0;
  static uint16_t count = 0;
  
  TCNT0 = TMR0_SET;
  
  if (m_flag & FLAG_ENTER) {
    m_flag ^= (FLAG_ENTER | FLAG_RUN);
	if (m_flag & FLAG_SPACE) {
	  beep = 0;
	  count = (m_flag & FLAG_LONG) ? DASH_SP :
	    ((m_flag & FLAG_WORD) ? WORD_SP : DOT);
	} else {
	  beep = 1;
	  count = (m_flag & FLAG_LONG) ? DASH : DOT;
	}	
  }

  if (m_flag & FLAG_RUN) {
	if (count) count--;
	  else {
	    m_flag = 0;
	    beep = 0;
	  }
  }
  
  if (beep) PORTD ^= O_SPK;
  
  return;
}


// vyslani retezce ulozeneho v RAM
void send(char *s)
{
  char *c = s;
  char chr;
  uint8_t lng, code, n;
  
  while (*c != '\0') {
    //chr = toupper(*c);
	chr = *c;
    // cislice
    if (chr >= '0' && chr <= '9') {
	  code = pgm_read_byte(morse_n + chr - '0');
	  lng = 5;
	} else {
	  // pismeno
	  if (chr >= 'A' && chr <= 'Z') {
	    code = pgm_read_byte(morse_c + chr - 'A');
	    lng = 1 + (code & 0x03);
	  } else {
	    // znak
		for (lng = 0, code = 0xFF; lng < 8; lng++) {
		  if (chr == pgm_read_byte(morse_zn + lng)) {
		    code = 0x00;
		    break;
		  }
		}
		if (code != 0xFF) {
		  code = pgm_read_byte(morse_zc + lng);
		  lng = 5 + (code & 0x01);
		} else {
		  // mezera nebo neznamy znak
		  Wait_TxFree();
          m_flag = FLAG_ENTER + FLAG_WORD_SP;
		  c++;
          continue; 
		}
	  }
	}
	
	// vyslani znaku
	for (chr = 0, n = 0x80; chr < lng; chr++, n >>= 1) {
	  Wait_TxFree();
      if (code & n) m_flag = FLAG_ENTER + FLAG_DASH;
	    else m_flag = FLAG_ENTER + FLAG_DOT;
	  Wait_TxFree();
	  if ((chr + 1) < lng) m_flag = FLAG_ENTER + FLAG_DOT_SP;
	    else m_flag = FLAG_ENTER + FLAG_DASH_SP;
	}
	// dalsi znak
    c++;
  }

  return;
}


// --- MAIN --------------------------------------------------------------------
#ifdef _DEBUG_
int main (void)
{
  volatile uint16_t i = 0;
  char *s = "TEST 12345";
  
  // casovac
  TCNT0 = TMR0_SET;
  TCCR0 = 0x04;
  TIMSK |= _BV (TOIE0);
  
  // port
  DDRD = O_SPK;
  PORTB = 0;
  
  sei();
  
  while (1) {
    send(s);
  
    for (i = 1; i < 20; i++) {
	  Wait_TxFree();
      m_flag = FLAG_ENTER + FLAG_WORD_SP;
	}
  }
}
#endif
// -----------------------------------------------------------------------------
