//-----------------------------------------------------------------------------
// morse.h - ver.1.0
// Modul pro vysilani Morseovy abecedy
// leden 2007
// WinAVR-20060421
// OK1CTR@seznam.cz http://home.zcu.cz/~rlinhart
//-----------------------------------------------------------------------------

#ifndef _MORSE_H_
#define _MORSE_H_

// --- HW KONFIGURACE MODULU ---------------------------------------------------

// nastaveni hlavniho casovace
#define XTAL       4000000  // krystal Hz
#define T0_PRE         256  // preddelic TMR0
#define FRQ_INT       1000  // frekvence preruseni v Hz

// io porty
#define O_SPK    0x18  // repracek

// --- KONFIGURACE APLIKACE ----------------------------------------------------

// morseovka
#define UNIT   60  // nejmensi interval - rastr
#define PARIS   1  // nasobek standardni delky mezer mezi pismeny

// --- POCITANE KONSTANTY A MAKRA ----------------------------------------------

// casovac
#define TMR0_SET (256 - (XTAL / T0_PRE) / FRQ_INT)

// intervaly v ticich casovace
#define DOT     morse_unit
#define DASH    (3 * DOT)  // delka carky
#define DASH_SP (DASH * morse_paris)  // delka mezery mezi pismeny
#define WORD_SP (2 * DASH_SP + DOT)  // delka mezer mezi slovy

// priznay a stavy automatu - vnitrni
#define FLAG_RUN      0x20  // prubeh vysilani znacky
#define FLAG_LONG     0x01  // interval mezipismenny
#define FLAG_WORD     0x04  // interval mezislovni
#define FLAG_SPACE    0x02  // interval bez zvuku

// priznay a stavy automatu - dotazovaci
#define FLAG_FREE     0x00  // volno, mozno vysilat

// priznay a stavy automatu - zadavaci
#define FLAG_ENTER    0x10  // vlozeni nove znacky
#define FLAG_DOT      0x00  // tecka
#define FLAG_DASH     0x01  // carka
#define FLAG_DOT_SP   0x02  // kratka mezera
#define FLAG_DASH_SP  0x03  // dlouha mezera
#define FLAG_WORD_SP  0x06  // mezera mezi slovy

// zisteni volneho vysilace
#define Wait_TxFree()  while (m_flag != FLAG_FREE)

// --- PROMENNE ----------------------------------------------------------------

// menitelne parametry
extern uint8_t morse_unit;
extern uint8_t morse_paris;

// rizeni pipadla
extern volatile uint8_t m_flag;

// --- FUNKCE ------------------------------------------------------------------

// vyslani retezce ulozeneho v RAM
void send(char *s);

// -----------------------------------------------------------------------------
#endif
