//-----------------------------------------------------------------------------
// trainer.c - ver.1.0
// Trenovaci program na Morseovu abecedu
// leden 2007
// WinAVR-20060421
// OK1CTR@seznam.cz http://home.zcu.cz/~rlinhart
//-----------------------------------------------------------------------------

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include "morse.h"

// --- HW KONFIGURACE MODULU ---------------------------------------------------

// spinace port B
#define KSW_B 0x87
// spinace port C
#define KSW_C 0x3F
// spinace port D
#define KSW_D 0xE0

// velikost EEPROM v Byte
#define EEPROM_MAX 512

// lekce inverzne, 1=lekce 1, 0=lekce 2
#define SW_LEKCE ((PINB & 0x02) >> 1)
// pocet znaku v lekci inverzne 0-31 = 1-32 (11111)-(00000)
#define SW_ZNAKU ((PINB & 0x01) | ((PIND & 0x80) >> 6) | ((PIND & 0x40) >> 4) | ((PIND & 0x20) >> 2) | ((PINB & 0x80) >> 3))
// rychlost inverzne, 0-15 (1111)-(0000)
#define SW_RYCHL ((PINB & 0x04) >> 2) | ((PINC & 0x07) << 1)
// ef.rychlost inverzne, 0-7 (111)-(000)
#define SW_PARIS ((PINC & 0x38) >> 3)

// --- DEFINICE LEKCI ----------------------------------------------------------

// lekce 1
#define B1_COUNT 30
prog_char bank1[] = "KMRSUAPTLOwI.NJEF0YV,G5/Q9ZH38";

// lekce 2
#define B2_COUNT 30
prog_char bank2[] = "wI.NJEF0YV,G5/Q9ZH38B?427C1D6X";

// minimalni delka trenovaci sekvence
#define MINSEQ 1
// maximalni delka trenovaci sekvence
#define MAXSEQ 5

// -----------------------------------------------------------------------------
// ulozi data pro nove prednastaveni generatoru
void set_seed(void)
{
  uint16_t i;
  unsigned int x;
  
  i = rand() & (EEPROM_MAX - 1);
  x = rand() & 0xFF;
  eeprom_write_byte((uint8_t *) i, x);
  return;
}

// -----------------------------------------------------------------------------
// prednastavi generator nahodnych cisel
unsigned int get_seed(void)
{
  uint16_t i;
  unsigned int x;
  
  for (i = 0, x = 0; i < EEPROM_MAX; i++) {
    x += eeprom_read_byte((uint8_t *) i);
  }
  x &= 0x7FFF;
  if (x == 0) x = 1;
  
  return(x);
}

// --- MAIN --------------------------------------------------------------------
int main (void)
{
  uint8_t count, sqlen, i;
  prog_char *bank;
  char s[MAXSEQ + 1];
  
  // casovac
  TCNT0 = TMR0_SET;
  TCCR0 = 0x04;
  TIMSK |= _BV (TOIE0);
  
  // pipaci port
  DDRD = O_SPK;
  PORTB = 0;
  
  // spinacove porty
  DDRB &= ~KSW_B;
  PORTB |= KSW_B;
  DDRC &= ~KSW_C;
  PORTC |= KSW_C;
  DDRD &= ~KSW_D;
  PORTD |= KSW_D;
  
  // nacitani vstupu - lekce
  count = 1 + ((SW_ZNAKU) ^ 0x1F);
  if ((SW_LEKCE) == 1) {
    bank = (char *) bank1;
	if (count > B1_COUNT) count = B1_COUNT;
  } else {
    bank = (char *) bank2;
	if (count > B2_COUNT) count = B2_COUNT;
  }
  
  // inicializace random generatoru
  srand(get_seed());
  
  sei();
  
  while (1) {
    // nacitani vstupu - rychlost a mezery
    morse_unit = (uint16_t)(1200 / (5 * ((SW_RYCHL) ^ 0x0F) + 15));
    morse_paris = 1 + ((SW_PARIS ^ 0x07));

    // generator sekvenci
	sqlen = MINSEQ + rand() % (MAXSEQ - MINSEQ + 1);
	for (i = 0; i < sqlen; i++) {
	  s[i] = pgm_read_byte(bank + rand() % count);
	}
	  s[i] = '\0';
	
	// vyslani sekvence
    send(s);
	
	// mezera mezi sekvencemi jako mezi slovy
	send(" ");
	
	// zapis noveho seminka pro random generator
	set_seed();
  }
}
// -----------------------------------------------------------------------------
